import pytest

from icupy.icu import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 62:
    pytest.skip("ICU4C<62", allow_module_level=True)

import copy

# fmt: off
from icupy.icu import (
    ICUError, Locale, MeasureUnit, StringEnumeration, UErrorCode,
)
from icupy.icu.number import NumberFormatter

# fmt: on


def test_api():
    # [1]
    # static int32_t icu::MeasureUnit::getAvailable(
    #       const char *type,
    #       MeasureUnit *destArray,
    #       int32_t destCapacity,
    #       UErrorCode &errorCode
    # )
    dest_array = MeasureUnit.get_available("currency")
    assert isinstance(dest_array, list)
    assert len(dest_array) > 0
    assert all(isinstance(x, MeasureUnit) for x in dest_array)
    assert all(x.get_type() == "currency" for x in dest_array)

    # [2]
    # static int32_t icu::MeasureUnit::getAvailable(
    #       MeasureUnit *destArray,
    #       int32_t destCapacity,
    #       UErrorCode &errorCode
    # )
    dest_array = MeasureUnit.get_available()
    assert isinstance(dest_array, list)
    assert len(dest_array) > 0
    assert all(isinstance(x, MeasureUnit) for x in dest_array)

    # static StringEnumeration *icu::MeasureUnit::getAvailableTypes(
    #       UErrorCode &errorCode
    # )
    types = MeasureUnit.get_available_types()
    assert isinstance(types, StringEnumeration)
    assert len(types) > 0
    assert "currency" in types

    # const char *icu::MeasureUnit::getSubtype()
    unit = MeasureUnit.create_square_kilometer()
    result = unit.get_subtype()
    assert isinstance(result, str)
    assert result == "square-kilometer"

    # const char *icu::MeasureUnit::getType()
    result = unit.get_type()
    assert isinstance(result, str)
    assert result == "area"

    # UBool icu::MeasureUnit::operator!=(const UObject &other)
    unit1 = MeasureUnit.get_kilometer()
    unit2 = unit1.clone()
    unit3 = MeasureUnit.get_kilometer_per_hour()
    assert not (unit1 != unit2)
    assert unit1 != unit3
    assert unit2 != unit3

    # UBool icu::MeasureUnit::operator==(const UObject &other)
    assert unit1 == unit2
    assert not (unit1 == unit3)
    assert not (unit2 == unit3)


def test_clone():
    unit1 = MeasureUnit.get_kilometer()

    # MeasureUnit *icu::MeasureUnit::clone()
    unit2 = unit1.clone()
    assert isinstance(unit2, MeasureUnit)
    assert unit2 == unit1

    unit3 = copy.copy(unit1)
    assert unit3 == unit1

    unit4 = copy.deepcopy(unit1)
    assert unit4 == unit1


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_for_identifier():
    # static MeasureUnit icu::MeasureUnit::forIdentifier(
    #       StringPiece identifier,
    #       UErrorCode &status
    # )
    unit = MeasureUnit.for_identifier("furlong-per-nanosecond")
    assert isinstance(unit, MeasureUnit)

    # const char *icu::MeasureUnit::getIdentifier()
    result = unit.get_identifier()
    assert isinstance(result, str)
    assert result == "furlong-per-nanosecond"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_get_complexity():
    from icupy.icu import UMeasureUnitComplexity

    # UMeasureUnitComplexity icu::MeasureUnit::getComplexity(
    #       UErrorCode &status
    # )
    unit = MeasureUnit.get_meter()
    assert unit.get_complexity() == UMeasureUnitComplexity.UMEASURE_UNIT_SINGLE

    unit = MeasureUnit.for_identifier("meter-per-second")
    assert (
        unit.get_complexity() == UMeasureUnitComplexity.UMEASURE_UNIT_COMPOUND
    )

    unit = MeasureUnit.for_identifier("hour-and-minute")
    assert unit.get_complexity() == UMeasureUnitComplexity.UMEASURE_UNIT_MIXED


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_get_dimensionality():
    # int32_t icu::MeasureUnit::getDimensionality(UErrorCode &status)
    unit = MeasureUnit.create_cubic_meter()
    assert unit.get_dimensionality() == 3

    unit = MeasureUnit.create_meter()
    assert unit.get_dimensionality() == 1


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_get_prefix():
    from icupy.icu import UMeasurePrefix

    # UMeasurePrefix icu::MeasureUnit::getPrefix(UErrorCode &status)
    unit = MeasureUnit.get_kilometer()
    assert unit.get_prefix() == UMeasurePrefix.UMEASURE_PREFIX_KILO


def test_measure_unit_53():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_acre()).format_int(1).to_string()
    ) == "1 ac"

    assert (
        fmt.unit(MeasureUnit.create_arc_minute()).format_int(1).to_string()
    ) == "1 arcmin"

    assert (
        fmt.unit(MeasureUnit.create_arc_second()).format_int(1).to_string()
    ) == "1 arcsec"

    assert (
        fmt.unit(MeasureUnit.create_celsius()).format_int(1).to_string()
    ) == "1\xb0C"  # 1°C

    assert (
        fmt.unit(MeasureUnit.create_centimeter()).format_int(1).to_string()
    ) == "1 cm"

    assert (
        fmt.unit(MeasureUnit.create_cubic_kilometer())
        .format_int(1)
        .to_string()
    ) == "1 km\xb3"  # 1 km³

    assert (
        fmt.unit(MeasureUnit.create_cubic_mile()).format_int(1).to_string()
    ) == "1 mi\xb3"  # 1 mi³

    assert (
        fmt.unit(MeasureUnit.create_day()).format_int(1).to_string()
    ) == "1 day"

    assert (
        fmt.unit(MeasureUnit.create_degree()).format_int(1).to_string()
    ) == "1 deg"

    assert (
        fmt.unit(MeasureUnit.create_fahrenheit()).format_int(1).to_string()
    ) == "1\xb0F"  # 1°F

    assert (
        fmt.unit(MeasureUnit.create_foot()).format_int(1).to_string()
    ) == "1 ft"

    assert (
        fmt.unit(MeasureUnit.create_gforce()).format_int(1).to_string()
    ) == "1 G"

    assert (
        fmt.unit(MeasureUnit.create_gram()).format_int(1).to_string()
    ) == "1 g"

    assert (
        fmt.unit(MeasureUnit.create_hectare()).format_int(1).to_string()
    ) == "1 ha"

    assert (
        fmt.unit(MeasureUnit.create_hectopascal()).format_int(1).to_string()
    ) == "1 hPa"

    assert (
        fmt.unit(MeasureUnit.create_horsepower()).format_int(1).to_string()
    ) == "1 hp"

    assert (
        fmt.unit(MeasureUnit.create_hour()).format_int(1).to_string()
    ) == "1 hr"

    assert (
        fmt.unit(MeasureUnit.create_inch()).format_int(1).to_string()
    ) == "1 in"

    assert (
        fmt.unit(MeasureUnit.create_inch_hg()).format_int(1).to_string()
    ) == "1 inHg"

    assert (
        fmt.unit(MeasureUnit.create_kilogram()).format_int(1).to_string()
    ) == "1 kg"

    assert (
        fmt.unit(MeasureUnit.create_kilometer()).format_int(1).to_string()
    ) == "1 km"

    assert (
        fmt.unit(MeasureUnit.create_kilometer_per_hour())
        .format_int(1)
        .to_string()
    ) == "1 km/h"

    assert (
        fmt.unit(MeasureUnit.create_kilowatt()).format_int(1).to_string()
    ) == "1 kW"

    assert (
        fmt.unit(MeasureUnit.create_light_year()).format_int(1).to_string()
    ) == "1 ly"

    assert (
        fmt.unit(MeasureUnit.create_liter()).format_int(1).to_string()
    ) == "1 L"

    assert (
        fmt.unit(MeasureUnit.create_meter()).format_int(1).to_string()
    ) == "1 m"

    assert (
        fmt.unit(MeasureUnit.create_meter_per_second())
        .format_int(1)
        .to_string()
    ) == "1 m/s"

    assert (
        fmt.unit(MeasureUnit.create_mile()).format_int(1).to_string()
    ) == "1 mi"

    assert (
        fmt.unit(MeasureUnit.create_mile_per_hour()).format_int(1).to_string()
    ) == "1 mph"

    assert (
        fmt.unit(MeasureUnit.create_millibar()).format_int(1).to_string()
    ) == "1 mbar"

    assert (
        fmt.unit(MeasureUnit.create_millimeter()).format_int(1).to_string()
    ) == "1 mm"

    assert (
        fmt.unit(MeasureUnit.create_millisecond()).format_int(1).to_string()
    ) == "1 ms"

    assert (
        fmt.unit(MeasureUnit.create_minute()).format_int(1).to_string()
    ) == "1 min"

    assert (
        fmt.unit(MeasureUnit.create_month()).format_int(1).to_string()
    ) == "1 mth"

    assert (
        fmt.unit(MeasureUnit.create_ounce()).format_int(1).to_string()
    ) == "1 oz"

    assert (
        fmt.unit(MeasureUnit.create_picometer()).format_int(1).to_string()
    ) == "1 pm"

    assert (
        fmt.unit(MeasureUnit.create_pound()).format_int(1).to_string()
    ) == "1 lb"

    assert (
        fmt.unit(MeasureUnit.create_second()).format_int(1).to_string()
    ) == "1 sec"

    assert (
        fmt.unit(MeasureUnit.create_square_foot()).format_int(1).to_string()
    ) == "1 sq ft"

    assert (
        fmt.unit(MeasureUnit.create_square_kilometer())
        .format_int(1)
        .to_string()
    ) == "1 km\xb2"  # 1 km²

    assert (
        fmt.unit(MeasureUnit.create_square_meter()).format_int(1).to_string()
    ) == "1 m\xb2"  # 1 m²

    assert (
        fmt.unit(MeasureUnit.create_square_mile()).format_int(1).to_string()
    ) == "1 sq mi"

    assert (
        fmt.unit(MeasureUnit.create_watt()).format_int(1).to_string()
    ) == "1 W"

    assert (
        fmt.unit(MeasureUnit.create_week()).format_int(1).to_string()
    ) == "1 wk"

    assert (
        fmt.unit(MeasureUnit.create_yard()).format_int(1).to_string()
    ) == "1 yd"

    assert (
        fmt.unit(MeasureUnit.create_year()).format_int(1).to_string()
    ) == "1 yr"


def test_measure_unit_54():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_acre_foot()).format_int(1).to_string()
    ) == "1 ac ft"

    assert (
        fmt.unit(MeasureUnit.create_ampere()).format_int(1).to_string()
    ) == "1 A"

    assert (
        fmt.unit(MeasureUnit.create_astronomical_unit())
        .format_int(1)
        .to_string()
    ) == "1 au"

    assert (
        fmt.unit(MeasureUnit.create_bit()).format_int(1).to_string()
    ) == "1 bit"

    assert (
        fmt.unit(MeasureUnit.create_bushel()).format_int(1).to_string()
    ) == "1 bu"

    assert (
        fmt.unit(MeasureUnit.create_byte()).format_int(1).to_string()
    ) == "1 byte"

    assert (
        fmt.unit(MeasureUnit.create_calorie()).format_int(1).to_string()
    ) == "1 cal"

    assert (
        fmt.unit(MeasureUnit.create_carat()).format_int(1).to_string()
    ) == "1 CD"

    assert (
        fmt.unit(MeasureUnit.create_centiliter()).format_int(1).to_string()
    ) == "1 cL"

    assert (
        fmt.unit(MeasureUnit.create_cubic_centimeter())
        .format_int(1)
        .to_string()
    ) == "1 cm\xb3"  # 1 cm³

    assert (
        fmt.unit(MeasureUnit.create_cubic_foot()).format_int(1).to_string()
    ) == "1 ft\xb3"  # 1 ft³

    assert (
        fmt.unit(MeasureUnit.create_cubic_inch()).format_int(1).to_string()
    ) == "1 in\xb3"  # 1 in³

    assert (
        fmt.unit(MeasureUnit.create_cubic_meter()).format_int(1).to_string()
    ) == "1 m\xb3"  # 1 m³

    assert (
        fmt.unit(MeasureUnit.create_cubic_yard()).format_int(1).to_string()
    ) == "1 yd\xb3"  # 1 yd³

    assert (
        fmt.unit(MeasureUnit.create_cup()).format_int(1).to_string()
    ) == "1 c"

    assert (
        fmt.unit(MeasureUnit.create_deciliter()).format_int(1).to_string()
    ) == "1 dL"

    assert (
        fmt.unit(MeasureUnit.create_decimeter()).format_int(1).to_string()
    ) == "1 dm"

    assert (
        fmt.unit(MeasureUnit.create_fathom()).format_int(1).to_string()
    ) == "1 fth"

    assert (
        fmt.unit(MeasureUnit.create_fluid_ounce()).format_int(1).to_string()
    ) == "1 fl oz"

    assert (
        fmt.unit(MeasureUnit.create_foodcalorie()).format_int(1).to_string()
    ) == "1 Cal"

    assert (
        fmt.unit(MeasureUnit.create_furlong()).format_int(1).to_string()
    ) == "1 fur"

    assert (
        fmt.unit(MeasureUnit.create_gallon()).format_int(1).to_string()
    ) == "1 gal"

    assert (
        fmt.unit(MeasureUnit.create_gigabit()).format_int(1).to_string()
    ) == "1 Gb"

    assert (
        fmt.unit(MeasureUnit.create_gigabyte()).format_int(1).to_string()
    ) == "1 GB"

    assert (
        fmt.unit(MeasureUnit.create_gigahertz()).format_int(1).to_string()
    ) == "1 GHz"

    assert (
        fmt.unit(MeasureUnit.create_gigawatt()).format_int(1).to_string()
    ) == "1 GW"

    assert (
        fmt.unit(MeasureUnit.create_hectoliter()).format_int(1).to_string()
    ) == "1 hL"

    assert (
        fmt.unit(MeasureUnit.create_hertz()).format_int(1).to_string()
    ) == "1 Hz"

    assert (
        fmt.unit(MeasureUnit.create_joule()).format_int(1).to_string()
    ) == "1 J"

    assert (
        fmt.unit(MeasureUnit.create_karat()).format_int(1).to_string()
    ) == "1 kt"

    assert (
        fmt.unit(MeasureUnit.create_kelvin()).format_int(1).to_string()
    ) == "1 K"

    assert (
        fmt.unit(MeasureUnit.create_kilobit()).format_int(1).to_string()
    ) == "1 kb"

    assert (
        fmt.unit(MeasureUnit.create_kilobyte()).format_int(1).to_string()
    ) == "1 kB"

    assert (
        fmt.unit(MeasureUnit.create_kilocalorie()).format_int(1).to_string()
    ) == "1 kcal"

    assert (
        fmt.unit(MeasureUnit.create_kilohertz()).format_int(1).to_string()
    ) == "1 kHz"

    assert (
        fmt.unit(MeasureUnit.create_kilojoule()).format_int(1).to_string()
    ) == "1 kJ"

    assert (
        fmt.unit(MeasureUnit.create_kilowatt_hour()).format_int(1).to_string()
    ) == "1 kWh"

    assert (
        fmt.unit(MeasureUnit.create_liter_per_kilometer())
        .format_int(1)
        .to_string()
    ) == "1 L/km"

    assert (
        fmt.unit(MeasureUnit.create_lux()).format_int(1).to_string()
    ) == "1 lx"

    assert (
        fmt.unit(MeasureUnit.create_megabit()).format_int(1).to_string()
    ) == "1 Mb"

    assert (
        fmt.unit(MeasureUnit.create_megabyte()).format_int(1).to_string()
    ) == "1 MB"

    assert (
        fmt.unit(MeasureUnit.create_megahertz()).format_int(1).to_string()
    ) == "1 MHz"

    assert (
        fmt.unit(MeasureUnit.create_megaliter()).format_int(1).to_string()
    ) == "1 ML"

    assert (
        fmt.unit(MeasureUnit.create_megawatt()).format_int(1).to_string()
    ) == "1 MW"

    assert (
        fmt.unit(MeasureUnit.create_meter_per_second_squared())
        .format_int(1)
        .to_string()
    ) == "1 m/s\xb2"  # 1 m/s²

    assert (
        fmt.unit(MeasureUnit.create_metric_ton()).format_int(1).to_string()
    ) == "1 t"

    assert (
        fmt.unit(MeasureUnit.create_microgram()).format_int(1).to_string()
    ) in (
        "1 \xb5g",
        "1 \u03bcg",
    )  # 1 µg

    assert (
        fmt.unit(MeasureUnit.create_micrometer()).format_int(1).to_string()
    ) in (
        "1 \xb5m",
        "1 \u03bcm",
    )  # 1 µm

    assert (
        fmt.unit(MeasureUnit.create_microsecond()).format_int(1).to_string()
    ) in (
        "1 \xb5s",
        "1 \u03bcs",
    )  # 1 µs

    assert (
        fmt.unit(MeasureUnit.create_mile_per_gallon())
        .format_int(1)
        .to_string()
    ) == "1 mpg"

    assert (
        fmt.unit(MeasureUnit.create_milliampere()).format_int(1).to_string()
    ) == "1 mA"

    assert (
        fmt.unit(MeasureUnit.create_milligram()).format_int(1).to_string()
    ) == "1 mg"

    assert (
        fmt.unit(MeasureUnit.create_milliliter()).format_int(1).to_string()
    ) == "1 mL"

    assert (
        fmt.unit(MeasureUnit.create_millimeter_of_mercury())
        .format_int(1)
        .to_string()
    ) == "1 mmHg"

    assert (
        fmt.unit(MeasureUnit.create_milliwatt()).format_int(1).to_string()
    ) == "1 mW"

    assert (
        fmt.unit(MeasureUnit.create_nanometer()).format_int(1).to_string()
    ) == "1 nm"

    assert (
        fmt.unit(MeasureUnit.create_nanosecond()).format_int(1).to_string()
    ) == "1 ns"

    assert (
        fmt.unit(MeasureUnit.create_nautical_mile()).format_int(1).to_string()
    ) == "1 nmi"

    assert (
        fmt.unit(MeasureUnit.create_ohm()).format_int(1).to_string()
    ) == "1 \u03a9"  # 1 Ω

    assert (
        fmt.unit(MeasureUnit.create_ounce_troy()).format_int(1).to_string()
    ) == "1 oz t"

    assert (
        fmt.unit(MeasureUnit.create_parsec()).format_int(1).to_string()
    ) == "1 pc"

    assert (
        fmt.unit(MeasureUnit.create_pint()).format_int(1).to_string()
    ) == "1 pt"

    assert (
        fmt.unit(MeasureUnit.create_pound_per_square_inch())
        .format_int(1)
        .to_string()
    ) == "1 psi"

    assert (
        fmt.unit(MeasureUnit.create_quart()).format_int(1).to_string()
    ) == "1 qt"

    assert (
        fmt.unit(MeasureUnit.create_radian()).format_int(1).to_string()
    ) == "1 rad"

    assert (
        fmt.unit(MeasureUnit.create_square_centimeter())
        .format_int(1)
        .to_string()
    ) == "1 cm\xb2"  # 1 cm²

    assert (
        fmt.unit(MeasureUnit.create_square_inch()).format_int(1).to_string()
    ) == "1 in\xb2"  # 1 in²

    assert (
        fmt.unit(MeasureUnit.create_square_yard()).format_int(1).to_string()
    ) == "1 yd\xb2"  # 1 yd²

    assert (
        fmt.unit(MeasureUnit.create_stone()).format_int(1).to_string()
    ) == "1 st"

    assert (
        fmt.unit(MeasureUnit.create_tablespoon()).format_int(1).to_string()
    ) == "1 tbsp"

    assert (
        fmt.unit(MeasureUnit.create_teaspoon()).format_int(1).to_string()
    ) == "1 tsp"

    assert (
        fmt.unit(MeasureUnit.create_terabit()).format_int(1).to_string()
    ) == "1 Tb"

    assert (
        fmt.unit(MeasureUnit.create_terabyte()).format_int(1).to_string()
    ) == "1 TB"

    assert (
        fmt.unit(MeasureUnit.create_ton()).format_int(1).to_string()
    ) == "1 tn"

    assert (
        fmt.unit(MeasureUnit.create_volt()).format_int(1).to_string()
    ) == "1 V"


def test_measure_unit_56():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_century()).format_int(1).to_string()
    ) == "1 c"

    assert (
        fmt.unit(MeasureUnit.create_cup_metric()).format_int(1).to_string()
    ) == "1 mc"

    assert (
        fmt.unit(MeasureUnit.create_generic_temperature())
        .format_int(1)
        .to_string()
    ) == "1\xb0"  # 1°

    assert (
        fmt.unit(MeasureUnit.create_knot()).format_int(1).to_string()
    ) == "1 kn"

    assert (
        fmt.unit(MeasureUnit.create_liter_per_100_kilometers())
        .format_int(1)
        .to_string()
    ) == "1 L/100 km"

    assert (
        fmt.unit(MeasureUnit.create_mile_scandinavian())
        .format_int(1)
        .to_string()
    ) == "1 smi"

    assert (
        fmt.unit(MeasureUnit.create_pint_metric()).format_int(1).to_string()
    ) == "1 mpt"

    assert (
        fmt.unit(MeasureUnit.create_revolution_angle())
        .format_int(1)
        .to_string()
    ) == "1 rev"


def test_measure_unit_57():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_gallon_imperial())
        .format_int(1)
        .to_string()
    ) == "1 gal Imp."

    assert (
        fmt.unit(MeasureUnit.create_mile_per_gallon_imperial())
        .format_int(1)
        .to_string()
    ) == "1 mpg Imp."

    try:
        assert (
            fmt.unit(MeasureUnit.create_milligram_per_deciliter())
            .format_int(1)
            .to_string()
        ) == "1 mg/dL"
    except ICUError as ex:
        assert ex.args[0] == UErrorCode.U_MISSING_RESOURCE_ERROR  # ICU 69.1

    assert (
        fmt.unit(MeasureUnit.create_millimole_per_liter())
        .format_int(1)
        .to_string()
    ) == "1 mmol/L"

    assert (
        fmt.unit(MeasureUnit.create_part_per_million())
        .format_int(1)
        .to_string()
    ) == "1 ppm"


def test_measure_unit_59():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_point()).format_int(1).to_string()
    ) == "1 pt"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_measure_unit_63():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_atmosphere()).format_int(1).to_string()
    ) == "1 atm"

    assert (
        fmt.unit(MeasureUnit.create_percent()).format_int(1).to_string()
    ) == "1%"

    assert (
        fmt.unit(MeasureUnit.create_permille()).format_int(1).to_string()
    ) == "1\u2030"  # 1‰

    assert (
        fmt.unit(MeasureUnit.create_petabyte()).format_int(1).to_string()
    ) == "1 PB"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_measure_unit_64():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_barrel()).format_int(1).to_string()
    ) == "1 bbl"

    assert (
        fmt.unit(MeasureUnit.create_british_thermal_unit())
        .format_int(1)
        .to_string()
    ) == "1 Btu"

    assert (
        fmt.unit(MeasureUnit.create_dalton()).format_int(1).to_string()
    ) == "1 Da"

    assert (
        fmt.unit(MeasureUnit.create_day_person()).format_int(1).to_string()
    ) == "1 day"

    assert (
        fmt.unit(MeasureUnit.create_dunam()).format_int(1).to_string()
    ) == "1 dunam"

    assert (
        fmt.unit(MeasureUnit.create_earth_mass()).format_int(1).to_string()
    ) == "1 M\u2295"  # 1 M⊕

    assert (
        fmt.unit(MeasureUnit.create_electronvolt()).format_int(1).to_string()
    ) == "1 eV"

    assert (
        fmt.unit(MeasureUnit.create_fluid_ounce_imperial())
        .format_int(1)
        .to_string()
    ) == "1 fl oz Imp."

    assert (
        fmt.unit(MeasureUnit.create_kilopascal()).format_int(1).to_string()
    ) == "1 kPa"

    assert (
        fmt.unit(MeasureUnit.create_megapascal()).format_int(1).to_string()
    ) == "1 MPa"

    assert (
        fmt.unit(MeasureUnit.create_mole()).format_int(1).to_string()
    ) == "1 mol"

    assert (
        fmt.unit(MeasureUnit.create_month_person()).format_int(1).to_string()
    ) == "1 mth"

    assert (
        fmt.unit(MeasureUnit.create_newton()).format_int(1).to_string()
    ) == "1 N"

    assert (
        fmt.unit(MeasureUnit.create_newton_meter()).format_int(1).to_string()
    ) == "1 N\u22c5m"  # 1 N⋅m

    assert (
        fmt.unit(MeasureUnit.create_permyriad()).format_int(1).to_string()
    ) == "1\u2031"  # 1‱

    assert (
        fmt.unit(MeasureUnit.create_pound_foot()).format_int(1).to_string()
    ) == "1 lbf\u22c5ft"  # 1 lb⋅ft

    assert (
        fmt.unit(MeasureUnit.create_pound_force()).format_int(1).to_string()
    ) == "1 lbf"

    assert (
        fmt.unit(MeasureUnit.create_solar_luminosity())
        .format_int(1)
        .to_string()
    ) == "1 L\u2609"  # 1 L☉

    assert (
        fmt.unit(MeasureUnit.create_solar_mass()).format_int(1).to_string()
    ) == "1 M\u2609"  # 1 M☉

    assert (
        fmt.unit(MeasureUnit.create_solar_radius()).format_int(1).to_string()
    ) == "1 R\u2609"  # 1 R☉

    assert (
        fmt.unit(MeasureUnit.create_week_person()).format_int(1).to_string()
    ) == "1 wk"

    assert (
        fmt.unit(MeasureUnit.create_year_person()).format_int(1).to_string()
    ) == "1 yr"

    assert (
        fmt.unit(MeasureUnit.get_acre()).format_int(1).to_string()
    ) == "1 ac"

    assert (
        fmt.unit(MeasureUnit.get_acre_foot()).format_int(1).to_string()
    ) == "1 ac ft"

    assert (
        fmt.unit(MeasureUnit.get_ampere()).format_int(1).to_string()
    ) == "1 A"

    assert (
        fmt.unit(MeasureUnit.get_arc_minute()).format_int(1).to_string()
    ) == "1 arcmin"

    assert (
        fmt.unit(MeasureUnit.get_arc_second()).format_int(1).to_string()
    ) == "1 arcsec"

    assert (
        fmt.unit(MeasureUnit.get_astronomical_unit()).format_int(1).to_string()
    ) == "1 au"

    assert (
        fmt.unit(MeasureUnit.get_atmosphere()).format_int(1).to_string()
    ) == "1 atm"

    assert (
        fmt.unit(MeasureUnit.get_barrel()).format_int(1).to_string()
    ) == "1 bbl"

    assert (
        fmt.unit(MeasureUnit.get_bit()).format_int(1).to_string()
    ) == "1 bit"

    assert (
        fmt.unit(MeasureUnit.get_british_thermal_unit())
        .format_int(1)
        .to_string()
    ) == "1 Btu"

    assert (
        fmt.unit(MeasureUnit.get_bushel()).format_int(1).to_string()
    ) == "1 bu"

    assert (
        fmt.unit(MeasureUnit.get_byte()).format_int(1).to_string()
    ) == "1 byte"

    assert (
        fmt.unit(MeasureUnit.get_calorie()).format_int(1).to_string()
    ) == "1 cal"

    assert (
        fmt.unit(MeasureUnit.get_carat()).format_int(1).to_string()
    ) == "1 CD"

    assert (
        fmt.unit(MeasureUnit.get_celsius()).format_int(1).to_string()
    ) == "1\xb0C"  # 1°C

    assert (
        fmt.unit(MeasureUnit.get_centiliter()).format_int(1).to_string()
    ) == "1 cL"

    assert (
        fmt.unit(MeasureUnit.get_centimeter()).format_int(1).to_string()
    ) == "1 cm"

    assert (
        fmt.unit(MeasureUnit.get_century()).format_int(1).to_string()
    ) == "1 c"

    assert (
        fmt.unit(MeasureUnit.get_cubic_centimeter()).format_int(1).to_string()
    ) == "1 cm\xb3"  # 1 cm³

    assert (
        fmt.unit(MeasureUnit.get_cubic_foot()).format_int(1).to_string()
    ) == "1 ft\xb3"  # 1 ft³

    assert (
        fmt.unit(MeasureUnit.get_cubic_inch()).format_int(1).to_string()
    ) == "1 in\xb3"  # 1 in³

    assert (
        fmt.unit(MeasureUnit.get_cubic_kilometer()).format_int(1).to_string()
    ) == "1 km\xb3"  # 1 km³

    assert (
        fmt.unit(MeasureUnit.get_cubic_meter()).format_int(1).to_string()
    ) == "1 m\xb3"  # 1 m³

    assert (
        fmt.unit(MeasureUnit.get_cubic_mile()).format_int(1).to_string()
    ) == "1 mi\xb3"  # 1 mi³

    assert (
        fmt.unit(MeasureUnit.get_cubic_yard()).format_int(1).to_string()
    ) == "1 yd\xb3"  # 1 yd³

    assert (fmt.unit(MeasureUnit.get_cup()).format_int(1).to_string()) == "1 c"

    assert (
        fmt.unit(MeasureUnit.get_cup_metric()).format_int(1).to_string()
    ) == "1 mc"

    assert (
        fmt.unit(MeasureUnit.get_dalton()).format_int(1).to_string()
    ) == "1 Da"

    assert (
        fmt.unit(MeasureUnit.get_day()).format_int(1).to_string()
    ) == "1 day"

    assert (
        fmt.unit(MeasureUnit.get_day_person()).format_int(1).to_string()
    ) == "1 day"

    assert (
        fmt.unit(MeasureUnit.get_deciliter()).format_int(1).to_string()
    ) == "1 dL"

    assert (
        fmt.unit(MeasureUnit.get_decimeter()).format_int(1).to_string()
    ) == "1 dm"

    assert (
        fmt.unit(MeasureUnit.get_degree()).format_int(1).to_string()
    ) == "1 deg"

    assert (
        fmt.unit(MeasureUnit.get_dunam()).format_int(1).to_string()
    ) == "1 dunam"

    assert (
        fmt.unit(MeasureUnit.get_earth_mass()).format_int(1).to_string()
    ) == "1 M\u2295"  # 1 M⊕

    assert (
        fmt.unit(MeasureUnit.get_electronvolt()).format_int(1).to_string()
    ) == "1 eV"

    assert (
        fmt.unit(MeasureUnit.get_fahrenheit()).format_int(1).to_string()
    ) == "1\xb0F"  # 1°F

    assert (
        fmt.unit(MeasureUnit.get_fathom()).format_int(1).to_string()
    ) == "1 fth"

    assert (
        fmt.unit(MeasureUnit.get_fluid_ounce()).format_int(1).to_string()
    ) == "1 fl oz"

    assert (
        fmt.unit(MeasureUnit.get_fluid_ounce_imperial())
        .format_int(1)
        .to_string()
    ) == "1 fl oz Imp."

    assert (
        fmt.unit(MeasureUnit.get_foodcalorie()).format_int(1).to_string()
    ) == "1 Cal"

    assert (
        fmt.unit(MeasureUnit.get_foot()).format_int(1).to_string()
    ) == "1 ft"

    assert (
        fmt.unit(MeasureUnit.get_furlong()).format_int(1).to_string()
    ) == "1 fur"

    assert (
        fmt.unit(MeasureUnit.get_gallon()).format_int(1).to_string()
    ) == "1 gal"

    assert (
        fmt.unit(MeasureUnit.get_gallon_imperial()).format_int(1).to_string()
    ) == "1 gal Imp."

    assert (
        fmt.unit(MeasureUnit.get_generic_temperature())
        .format_int(1)
        .to_string()
    ) == "1\xb0"  # 1°

    assert (
        fmt.unit(MeasureUnit.get_gforce()).format_int(1).to_string()
    ) == "1 G"

    assert (
        fmt.unit(MeasureUnit.get_gigabit()).format_int(1).to_string()
    ) == "1 Gb"

    assert (
        fmt.unit(MeasureUnit.get_gigabyte()).format_int(1).to_string()
    ) == "1 GB"

    assert (
        fmt.unit(MeasureUnit.get_gigahertz()).format_int(1).to_string()
    ) == "1 GHz"

    assert (
        fmt.unit(MeasureUnit.get_gigawatt()).format_int(1).to_string()
    ) == "1 GW"

    assert (
        fmt.unit(MeasureUnit.get_gram()).format_int(1).to_string()
    ) == "1 g"

    assert (
        fmt.unit(MeasureUnit.get_hectare()).format_int(1).to_string()
    ) == "1 ha"

    assert (
        fmt.unit(MeasureUnit.get_hectoliter()).format_int(1).to_string()
    ) == "1 hL"

    assert (
        fmt.unit(MeasureUnit.get_hectopascal()).format_int(1).to_string()
    ) == "1 hPa"

    assert (
        fmt.unit(MeasureUnit.get_hertz()).format_int(1).to_string()
    ) == "1 Hz"

    assert (
        fmt.unit(MeasureUnit.get_horsepower()).format_int(1).to_string()
    ) == "1 hp"

    assert (
        fmt.unit(MeasureUnit.get_hour()).format_int(1).to_string()
    ) == "1 hr"

    assert (
        fmt.unit(MeasureUnit.get_inch()).format_int(1).to_string()
    ) == "1 in"

    assert (
        fmt.unit(MeasureUnit.get_inch_hg()).format_int(1).to_string()
    ) == "1 inHg"

    assert (
        fmt.unit(MeasureUnit.get_joule()).format_int(1).to_string()
    ) == "1 J"

    assert (
        fmt.unit(MeasureUnit.get_karat()).format_int(1).to_string()
    ) == "1 kt"

    assert (
        fmt.unit(MeasureUnit.get_kelvin()).format_int(1).to_string()
    ) == "1 K"

    assert (
        fmt.unit(MeasureUnit.get_kilobit()).format_int(1).to_string()
    ) == "1 kb"

    assert (
        fmt.unit(MeasureUnit.get_kilobyte()).format_int(1).to_string()
    ) == "1 kB"

    assert (
        fmt.unit(MeasureUnit.get_kilocalorie()).format_int(1).to_string()
    ) == "1 kcal"

    assert (
        fmt.unit(MeasureUnit.get_kilogram()).format_int(1).to_string()
    ) == "1 kg"

    assert (
        fmt.unit(MeasureUnit.get_kilohertz()).format_int(1).to_string()
    ) == "1 kHz"

    assert (
        fmt.unit(MeasureUnit.get_kilojoule()).format_int(1).to_string()
    ) == "1 kJ"

    assert (
        fmt.unit(MeasureUnit.get_kilometer()).format_int(1).to_string()
    ) == "1 km"

    assert (
        fmt.unit(MeasureUnit.get_kilometer_per_hour())
        .format_int(1)
        .to_string()
    ) == "1 km/h"

    assert (
        fmt.unit(MeasureUnit.get_kilopascal()).format_int(1).to_string()
    ) == "1 kPa"

    assert (
        fmt.unit(MeasureUnit.get_kilowatt()).format_int(1).to_string()
    ) == "1 kW"

    assert (
        fmt.unit(MeasureUnit.get_kilowatt_hour()).format_int(1).to_string()
    ) == "1 kWh"

    assert (
        fmt.unit(MeasureUnit.get_knot()).format_int(1).to_string()
    ) == "1 kn"

    assert (
        fmt.unit(MeasureUnit.get_light_year()).format_int(1).to_string()
    ) == "1 ly"

    assert (
        fmt.unit(MeasureUnit.get_liter()).format_int(1).to_string()
    ) == "1 L"

    assert (
        fmt.unit(MeasureUnit.get_liter_per_100_kilometers())
        .format_int(1)
        .to_string()
    ) == "1 L/100 km"

    assert (
        fmt.unit(MeasureUnit.get_liter_per_kilometer())
        .format_int(1)
        .to_string()
    ) == "1 L/km"

    assert (
        fmt.unit(MeasureUnit.get_lux()).format_int(1).to_string()
    ) == "1 lx"

    assert (
        fmt.unit(MeasureUnit.get_megabit()).format_int(1).to_string()
    ) == "1 Mb"

    assert (
        fmt.unit(MeasureUnit.get_megabyte()).format_int(1).to_string()
    ) == "1 MB"

    assert (
        fmt.unit(MeasureUnit.get_megahertz()).format_int(1).to_string()
    ) == "1 MHz"

    assert (
        fmt.unit(MeasureUnit.get_megaliter()).format_int(1).to_string()
    ) == "1 ML"

    assert (
        fmt.unit(MeasureUnit.get_megapascal()).format_int(1).to_string()
    ) == "1 MPa"

    assert (
        fmt.unit(MeasureUnit.get_megawatt()).format_int(1).to_string()
    ) == "1 MW"

    assert (
        fmt.unit(MeasureUnit.get_meter()).format_int(1).to_string()
    ) == "1 m"

    assert (
        fmt.unit(MeasureUnit.get_meter_per_second()).format_int(1).to_string()
    ) == "1 m/s"

    assert (
        fmt.unit(MeasureUnit.get_meter_per_second_squared())
        .format_int(1)
        .to_string()
    ) == "1 m/s\xb2"  # 1 m/s²

    assert (
        fmt.unit(MeasureUnit.get_metric_ton()).format_int(1).to_string()
    ) == "1 t"

    assert (
        fmt.unit(MeasureUnit.get_microgram()).format_int(1).to_string()
    ) in (
        "1 \xb5g",
        "1 \u03bcg",
    )  # 1 µg

    assert (
        fmt.unit(MeasureUnit.get_micrometer()).format_int(1).to_string()
    ) in (
        "1 \xb5m",
        "1 \u03bcm",
    )  # 1 µm

    assert (
        fmt.unit(MeasureUnit.get_microsecond()).format_int(1).to_string()
    ) in (
        "1 \xb5s",
        "1 \u03bcs",
    )  # 1 µs

    assert (
        fmt.unit(MeasureUnit.get_mile()).format_int(1).to_string()
    ) == "1 mi"

    assert (
        fmt.unit(MeasureUnit.get_mile_per_gallon()).format_int(1).to_string()
    ) == "1 mpg"

    assert (
        fmt.unit(MeasureUnit.get_mile_per_gallon_imperial())
        .format_int(1)
        .to_string()
    ) == "1 mpg Imp."

    assert (
        fmt.unit(MeasureUnit.get_mile_per_hour()).format_int(1).to_string()
    ) == "1 mph"

    assert (
        fmt.unit(MeasureUnit.get_mile_scandinavian()).format_int(1).to_string()
    ) == "1 smi"

    assert (
        fmt.unit(MeasureUnit.get_milliampere()).format_int(1).to_string()
    ) == "1 mA"

    assert (
        fmt.unit(MeasureUnit.get_millibar()).format_int(1).to_string()
    ) == "1 mbar"

    assert (
        fmt.unit(MeasureUnit.get_milligram()).format_int(1).to_string()
    ) == "1 mg"

    try:
        assert (
            fmt.unit(MeasureUnit.get_milligram_per_deciliter())
            .format_int(1)
            .to_string()
        ) == "1 mg/dL"
    except ICUError as ex:
        assert ex.args[0] == UErrorCode.U_MISSING_RESOURCE_ERROR  # ICU 69.1

    assert (
        fmt.unit(MeasureUnit.get_milliliter()).format_int(1).to_string()
    ) == "1 mL"

    assert (
        fmt.unit(MeasureUnit.get_millimeter()).format_int(1).to_string()
    ) == "1 mm"

    assert (
        fmt.unit(MeasureUnit.get_millimeter_of_mercury())
        .format_int(1)
        .to_string()
    ) == "1 mmHg"

    assert (
        fmt.unit(MeasureUnit.get_millimole_per_liter())
        .format_int(1)
        .to_string()
    ) == "1 mmol/L"

    assert (
        fmt.unit(MeasureUnit.get_millisecond()).format_int(1).to_string()
    ) == "1 ms"

    assert (
        fmt.unit(MeasureUnit.get_milliwatt()).format_int(1).to_string()
    ) == "1 mW"

    assert (
        fmt.unit(MeasureUnit.get_minute()).format_int(1).to_string()
    ) == "1 min"

    assert (
        fmt.unit(MeasureUnit.get_mole()).format_int(1).to_string()
    ) == "1 mol"

    assert (
        fmt.unit(MeasureUnit.get_month()).format_int(1).to_string()
    ) == "1 mth"

    assert (
        fmt.unit(MeasureUnit.get_month_person()).format_int(1).to_string()
    ) == "1 mth"

    assert (
        fmt.unit(MeasureUnit.get_nanometer()).format_int(1).to_string()
    ) == "1 nm"

    assert (
        fmt.unit(MeasureUnit.get_nanosecond()).format_int(1).to_string()
    ) == "1 ns"

    assert (
        fmt.unit(MeasureUnit.get_nautical_mile()).format_int(1).to_string()
    ) == "1 nmi"

    assert (
        fmt.unit(MeasureUnit.get_newton()).format_int(1).to_string()
    ) == "1 N"

    assert (
        fmt.unit(MeasureUnit.get_newton_meter()).format_int(1).to_string()
    ) == "1 N\u22c5m"  # 1 N⋅m

    assert (
        fmt.unit(MeasureUnit.get_ohm()).format_int(1).to_string()
    ) == "1 \u03a9"  # 1 Ω

    assert (
        fmt.unit(MeasureUnit.get_ounce()).format_int(1).to_string()
    ) == "1 oz"

    assert (
        fmt.unit(MeasureUnit.get_ounce_troy()).format_int(1).to_string()
    ) == "1 oz t"

    assert (
        fmt.unit(MeasureUnit.get_parsec()).format_int(1).to_string()
    ) == "1 pc"

    assert (
        fmt.unit(MeasureUnit.get_part_per_million()).format_int(1).to_string()
    ) == "1 ppm"

    assert (
        fmt.unit(MeasureUnit.get_percent()).format_int(1).to_string()
    ) == "1%"

    assert (
        fmt.unit(MeasureUnit.get_permille()).format_int(1).to_string()
    ) == "1\u2030"  # 1‰

    assert (
        fmt.unit(MeasureUnit.get_permyriad()).format_int(1).to_string()
    ) == "1\u2031"  # 1‱

    assert (
        fmt.unit(MeasureUnit.get_petabyte()).format_int(1).to_string()
    ) == "1 PB"

    assert (
        fmt.unit(MeasureUnit.get_picometer()).format_int(1).to_string()
    ) == "1 pm"

    assert (
        fmt.unit(MeasureUnit.get_pint()).format_int(1).to_string()
    ) == "1 pt"

    assert (
        fmt.unit(MeasureUnit.get_pint_metric()).format_int(1).to_string()
    ) == "1 mpt"

    assert (
        fmt.unit(MeasureUnit.get_point()).format_int(1).to_string()
    ) == "1 pt"

    assert (
        fmt.unit(MeasureUnit.get_pound()).format_int(1).to_string()
    ) == "1 lb"

    assert (
        fmt.unit(MeasureUnit.get_pound_foot()).format_int(1).to_string()
    ) == "1 lbf\u22c5ft"  # 1 lb⋅ft

    assert (
        fmt.unit(MeasureUnit.get_pound_force()).format_int(1).to_string()
    ) == "1 lbf"

    assert (
        fmt.unit(MeasureUnit.get_pound_per_square_inch())
        .format_int(1)
        .to_string()
    ) == "1 psi"

    assert (
        fmt.unit(MeasureUnit.get_quart()).format_int(1).to_string()
    ) == "1 qt"

    assert (
        fmt.unit(MeasureUnit.get_radian()).format_int(1).to_string()
    ) == "1 rad"

    assert (
        fmt.unit(MeasureUnit.get_revolution_angle()).format_int(1).to_string()
    ) == "1 rev"

    assert (
        fmt.unit(MeasureUnit.get_second()).format_int(1).to_string()
    ) == "1 sec"

    assert (
        fmt.unit(MeasureUnit.get_solar_luminosity()).format_int(1).to_string()
    ) == "1 L\u2609"  # 1 L☉

    assert (
        fmt.unit(MeasureUnit.get_solar_mass()).format_int(1).to_string()
    ) == "1 M\u2609"  # 1 M☉

    assert (
        fmt.unit(MeasureUnit.get_solar_radius()).format_int(1).to_string()
    ) == "1 R\u2609"  # 1 R☉

    assert (
        fmt.unit(MeasureUnit.get_square_centimeter()).format_int(1).to_string()
    ) == "1 cm\xb2"  # 1 cm²

    assert (
        fmt.unit(MeasureUnit.get_square_foot()).format_int(1).to_string()
    ) == "1 sq ft"

    assert (
        fmt.unit(MeasureUnit.get_square_inch()).format_int(1).to_string()
    ) == "1 in\xb2"  # 1 in²

    assert (
        fmt.unit(MeasureUnit.get_square_kilometer()).format_int(1).to_string()
    ) == "1 km\xb2"  # 1 km²

    assert (
        fmt.unit(MeasureUnit.get_square_meter()).format_int(1).to_string()
    ) == "1 m\xb2"  # 1 m²

    assert (
        fmt.unit(MeasureUnit.get_square_mile()).format_int(1).to_string()
    ) == "1 sq mi"

    assert (
        fmt.unit(MeasureUnit.get_square_yard()).format_int(1).to_string()
    ) == "1 yd\xb2"  # 1 yd²

    assert (
        fmt.unit(MeasureUnit.get_stone()).format_int(1).to_string()
    ) == "1 st"

    assert (
        fmt.unit(MeasureUnit.get_tablespoon()).format_int(1).to_string()
    ) == "1 tbsp"

    assert (
        fmt.unit(MeasureUnit.get_teaspoon()).format_int(1).to_string()
    ) == "1 tsp"

    assert (
        fmt.unit(MeasureUnit.get_terabit()).format_int(1).to_string()
    ) == "1 Tb"

    assert (
        fmt.unit(MeasureUnit.get_terabyte()).format_int(1).to_string()
    ) == "1 TB"

    assert (
        fmt.unit(MeasureUnit.get_ton()).format_int(1).to_string()
    ) == "1 tn"

    assert (
        fmt.unit(MeasureUnit.get_volt()).format_int(1).to_string()
    ) == "1 V"

    assert (
        fmt.unit(MeasureUnit.get_watt()).format_int(1).to_string()
    ) == "1 W"

    assert (
        fmt.unit(MeasureUnit.get_week()).format_int(1).to_string()
    ) == "1 wk"

    assert (
        fmt.unit(MeasureUnit.get_week_person()).format_int(1).to_string()
    ) == "1 wk"

    assert (
        fmt.unit(MeasureUnit.get_yard()).format_int(1).to_string()
    ) == "1 yd"

    assert (
        fmt.unit(MeasureUnit.get_year()).format_int(1).to_string()
    ) == "1 yr"

    assert (
        fmt.unit(MeasureUnit.get_year_person()).format_int(1).to_string()
    ) == "1 yr"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 65, reason="ICU4C<65")
def test_measure_unit_65():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_bar()).format_int(1).to_string()
    ) == "1 bar"

    assert (
        fmt.unit(MeasureUnit.create_decade()).format_int(1).to_string()
    ) == "1 dec"

    assert (
        fmt.unit(MeasureUnit.create_dot_per_centimeter())
        .format_int(1)
        .to_string()
    ) == "1 dpcm"

    assert (
        fmt.unit(MeasureUnit.create_dot_per_inch()).format_int(1).to_string()
    ) == "1 dpi"

    assert (
        fmt.unit(MeasureUnit.create_em()).format_int(1).to_string()
    ) == "1 em"

    assert (
        fmt.unit(MeasureUnit.create_megapixel()).format_int(1).to_string()
    ) == "1 MP"

    assert (
        fmt.unit(MeasureUnit.create_pascal()).format_int(1).to_string()
    ) == "1 Pa"

    assert (
        fmt.unit(MeasureUnit.create_pixel()).format_int(1).to_string()
    ) == "1 px"

    assert (
        fmt.unit(MeasureUnit.create_pixel_per_centimeter())
        .format_int(1)
        .to_string()
    ) == "1 ppcm"

    assert (
        fmt.unit(MeasureUnit.create_pixel_per_inch()).format_int(1).to_string()
    ) == "1 ppi"

    assert (
        fmt.unit(MeasureUnit.create_therm_us()).format_int(1).to_string()
    ) == "1 US therm"

    assert (
        fmt.unit(MeasureUnit.get_bar()).format_int(1).to_string()
    ) == "1 bar"

    assert (
        fmt.unit(MeasureUnit.get_decade()).format_int(1).to_string()
    ) == "1 dec"

    assert (
        fmt.unit(MeasureUnit.get_dot_per_centimeter())
        .format_int(1)
        .to_string()
    ) == "1 dpcm"

    assert (
        fmt.unit(MeasureUnit.get_dot_per_inch()).format_int(1).to_string()
    ) == "1 dpi"

    assert (fmt.unit(MeasureUnit.get_em()).format_int(1).to_string()) == "1 em"

    assert (
        fmt.unit(MeasureUnit.get_megapixel()).format_int(1).to_string()
    ) == "1 MP"

    assert (
        fmt.unit(MeasureUnit.get_pascal()).format_int(1).to_string()
    ) == "1 Pa"

    assert (
        fmt.unit(MeasureUnit.get_pixel()).format_int(1).to_string()
    ) == "1 px"

    assert (
        fmt.unit(MeasureUnit.get_pixel_per_centimeter())
        .format_int(1)
        .to_string()
    ) == "1 ppcm"

    assert (
        fmt.unit(MeasureUnit.get_pixel_per_inch()).format_int(1).to_string()
    ) == "1 ppi"

    assert (
        fmt.unit(MeasureUnit.get_therm_us()).format_int(1).to_string()
    ) == "1 US therm"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_measure_unit_68():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_candela()).format_int(1).to_string()
    ) == "1 cd"

    assert (
        fmt.unit(MeasureUnit.create_dessert_spoon()).format_int(1).to_string()
    ) == "1 dstspn"

    assert (
        fmt.unit(MeasureUnit.create_dessert_spoon_imperial())
        .format_int(1)
        .to_string()
    ) == "1 dstspn Imp"

    assert (fmt.unit(MeasureUnit.create_dot()).format_int(1).to_string()) in (
        "1 dot",  # ICU4C<69
        "1 px",  # ICU4C>=69
    )

    assert (
        fmt.unit(MeasureUnit.create_dram()).format_int(1).to_string()
    ) == "1 dram fl"

    assert (
        fmt.unit(MeasureUnit.create_drop()).format_int(1).to_string()
    ) == "1 drop"

    assert (
        fmt.unit(MeasureUnit.create_earth_radius()).format_int(1).to_string()
    ) == "1 R\u2295"  # 1 R⊕

    assert (
        fmt.unit(MeasureUnit.create_grain()).format_int(1).to_string()
    ) == "1 grain"

    assert (
        fmt.unit(MeasureUnit.create_jigger()).format_int(1).to_string()
    ) == "1 jigger"

    assert (
        fmt.unit(MeasureUnit.create_lumen()).format_int(1).to_string()
    ) == "1 lm"

    assert (
        fmt.unit(MeasureUnit.create_pinch()).format_int(1).to_string()
    ) == "1 pinch"

    assert (
        fmt.unit(MeasureUnit.create_quart_imperial()).format_int(1).to_string()
    ) == "1 qt Imp."

    assert (
        fmt.unit(MeasureUnit.get_candela()).format_int(1).to_string()
    ) == "1 cd"

    assert (
        fmt.unit(MeasureUnit.get_dessert_spoon()).format_int(1).to_string()
    ) == "1 dstspn"

    assert (
        fmt.unit(MeasureUnit.get_dessert_spoon_imperial())
        .format_int(1)
        .to_string()
    ) == "1 dstspn Imp"

    assert (fmt.unit(MeasureUnit.get_dot()).format_int(1).to_string()) in (
        "1 dot",  # ICU4C<69
        "1 px",
    )  # ICU4C>=69

    assert (
        fmt.unit(MeasureUnit.get_dram()).format_int(1).to_string()
    ) == "1 dram fl"

    assert (
        fmt.unit(MeasureUnit.get_drop()).format_int(1).to_string()
    ) == "1 drop"

    assert (
        fmt.unit(MeasureUnit.get_earth_radius()).format_int(1).to_string()
    ) == "1 R\u2295"  # 1 R⊕

    assert (
        fmt.unit(MeasureUnit.get_grain()).format_int(1).to_string()
    ) == "1 grain"

    assert (
        fmt.unit(MeasureUnit.get_jigger()).format_int(1).to_string()
    ) == "1 jigger"

    assert (
        fmt.unit(MeasureUnit.get_lumen()).format_int(1).to_string()
    ) == "1 lm"

    assert (
        fmt.unit(MeasureUnit.get_pinch()).format_int(1).to_string()
    ) == "1 pinch"

    assert (
        fmt.unit(MeasureUnit.get_quart_imperial()).format_int(1).to_string()
    ) == "1 qt Imp."


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_measure_unit_69():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    assert (
        fmt.unit(MeasureUnit.create_milligram_of_glucose_per_deciliter())
        .format_int(1)
        .to_string()
    ) == "1 mg/dL"

    assert (
        fmt.unit(MeasureUnit.get_milligram_of_glucose_per_deciliter())
        .format_int(1)
        .to_string()
    ) == "1 mg/dL"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 70, reason="ICU4C<70")
def test_measure_unit_70():
    fmt = NumberFormatter.with_locale(Locale.get_us())

    # static MeasureUnit *icu::MeasureUnit::createItem(UErrorCode &status)
    assert (
        fmt.unit(MeasureUnit.create_item()).format_int(1).to_string()
    ) == "1 item"

    # static MeasureUnit *icu::MeasureUnit::createKilowattHourPer100Kilometer(
    #       UErrorCode &status
    # )
    assert (
        fmt.unit(MeasureUnit.create_kilowatt_hour_per_100_kilometer())
        .format_int(1)
        .to_string()
    ) == "1 kWh/100km"

    # static MeasureUnit icu::MeasureUnit::getItem()
    assert (
        fmt.unit(MeasureUnit.get_item()).format_int(1).to_string()
    ) == "1 item"

    # static MeasureUnit icu::MeasureUnit::getKilowattHourPer100Kilometer()
    assert (
        fmt.unit(MeasureUnit.get_kilowatt_hour_per_100_kilometer())
        .format_int(1)
        .to_string()
    ) == "1 kWh/100km"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_product():
    # MeasureUnit icu::MeasureUnit::product(
    #       const MeasureUnit &other,
    #       UErrorCode &status
    # )
    unit = MeasureUnit.for_identifier("kilowatt")
    unit2 = unit.product(MeasureUnit.for_identifier("hour-per-day"))
    assert isinstance(unit2, MeasureUnit)
    assert unit2.get_identifier() in (
        "hour-kilowatt-per-day",  # ICU4C<69
        "kilowatt-hour-per-day",  # ICU4C>=69
    )


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_reciprocal():
    # MeasureUnit icu::MeasureUnit::reciprocal(UErrorCode &status)
    unit = MeasureUnit.for_identifier("meter-per-second")
    unit2 = unit.reciprocal()
    assert isinstance(unit2, MeasureUnit)
    assert unit2.get_identifier() == "second-per-meter"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_split_to_single_units():
    unit = MeasureUnit.create_newton_meter()

    # std::pair<LocalArray<MeasureUnit>, int32_t>
    # icu::MeasureUnit::splitToSingleUnits(UErrorCode &status)
    units = unit.split_to_single_units()
    assert isinstance(units, list)
    assert len(units) == 2
    assert all(isinstance(x, MeasureUnit) for x in units)
    assert units[0].get_identifier() == "newton"
    assert units[1].get_identifier() == "meter"

    unit = MeasureUnit.create_kilometer()
    units = unit.split_to_single_units()
    assert isinstance(units, list)
    assert len(units) == 1
    assert all(isinstance(x, MeasureUnit) for x in units)
    assert units[0].get_identifier() == "kilometer"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 67, reason="ICU4C<67")
def test_with_dimensionality():
    unit = MeasureUnit.create_kilometer()

    # MeasureUnit icu::MeasureUnit::withDimensionality(
    #       int32_t dimensionality,
    #       UErrorCode &status
    # )
    unit2 = unit.with_dimensionality(2)
    assert isinstance(unit2, MeasureUnit)
    assert unit2.get_identifier() == "square-kilometer"

    unit3 = unit.with_dimensionality(3)
    assert isinstance(unit3, MeasureUnit)
    assert unit3.get_identifier() == "cubic-kilometer"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_with_prefix():
    from icupy.icu import UMeasurePrefix

    unit = MeasureUnit.create_byte()

    # MeasureUnit icu::MeasureUnit::withPrefix(
    #       UMeasurePrefix prefix,
    #       UErrorCode &status
    # )
    unit2 = unit.with_prefix(UMeasurePrefix.UMEASURE_PREFIX_KILO)
    assert isinstance(unit2, MeasureUnit)
    assert unit2.get_identifier() == "kilobyte"



# fmt: off
from icupy.icu import (
    USET_ADD_CASE_MAPPINGS, ULocaleDataDelimiterType,
    ULocaleDataExemplarSetType, UMeasurementSystem, UnicodeSet, ulocdata_close,
    ulocdata_get_cldr_version, ulocdata_get_delimiter,
    ulocdata_get_exemplar_set, ulocdata_get_locale_display_pattern,
    ulocdata_get_locale_separator, ulocdata_get_measurement_system,
    ulocdata_get_no_substitute, ulocdata_get_paper_size, ulocdata_open,
    ulocdata_set_no_substitute,
)
from icupy.utils import gc

# fmt: on


def test_api():
    # void ulocdata_getCLDRVersion(
    #       UVersionInfo versionArray,
    #       UErrorCode *status
    # )
    version_array = ulocdata_get_cldr_version()
    assert isinstance(version_array, list)
    assert len(version_array) > 0
    assert all(isinstance(x, int) for x in version_array)

    # UMeasurementSystem ulocdata_getMeasurementSystem(
    #       const char *localeID,
    #       UErrorCode *status
    # )
    assert ulocdata_get_measurement_system("en") == UMeasurementSystem.UMS_US

    # void ulocdata_getPaperSize(
    #       const char *localeID,
    #       int32_t *height,
    #       int32_t *width,
    #       UErrorCode *status
    # )
    height, width = ulocdata_get_paper_size("en")
    assert (height, width) == (279, 216)

    # ULocaleData *ulocdata_open(
    #       const char *localeID,
    #       UErrorCode *status
    # )
    # void ulocdata_close(LocaleData *uld)
    with gc(ulocdata_open("en"), ulocdata_close) as uld:
        # int32_t ulocdata_getDelimiter(
        #       ULocaleData *uld,
        #       ULocaleDataDelimiterType type,
        #       UChar *result,
        #       int32_t resultLength,
        #       UErrorCode *status
        # )
        result = ulocdata_get_delimiter(
            uld, ULocaleDataDelimiterType.ULOCDATA_QUOTATION_START
        )
        assert isinstance(result, str)
        assert result == "\u201c"  # '“'

        # USet *ulocdata_getExemplarSet(
        #       ULocaleData *uld,
        #       USet *fillIn,
        #       uint32_t options,
        #       ULocaleDataExemplarSetType extype,
        #       UErrorCode *status
        # )
        uset1 = ulocdata_get_exemplar_set(
            uld,
            None,
            USET_ADD_CASE_MAPPINGS,
            ULocaleDataExemplarSetType.ULOCDATA_ES_STANDARD,
        )
        uniset1 = UnicodeSet.from_uset(uset1)
        assert uniset1.size() > 0

        uniset2 = UnicodeSet()
        fill_in = uniset2.to_uset()
        uset3 = ulocdata_get_exemplar_set(
            uld,
            fill_in,
            USET_ADD_CASE_MAPPINGS,
            ULocaleDataExemplarSetType.ULOCDATA_ES_STANDARD,
        )
        uniset3 = UnicodeSet.from_uset(uset3)
        assert uniset1 == uniset2 == uniset3

        # int32_t ulocdata_getLocaleDisplayPattern(
        #       ULocaleData *uld,
        #       UChar *pattern,
        #       int32_t patternCapacity,
        #       UErrorCode *status
        # )
        pattern = ulocdata_get_locale_display_pattern(uld)
        assert isinstance(pattern, str)
        assert pattern == "{0} ({1})"

        # int32_t ulocdata_getLocaleSeparator(
        #       ULocaleData *uld,
        #       UChar *separator,
        #       int32_t separatorCapacity,
        #       UErrorCode *status
        # )
        separator = ulocdata_get_locale_separator(uld)
        assert isinstance(separator, str)
        assert separator == ", "

        # UBool ulocdata_getNoSubstitute(ULocaleData *uld)
        assert not ulocdata_get_no_substitute(uld)

        # void ulocdata_setNoSubstitute(
        #       ULocaleData *uld,
        #       UBool setting
        # )
        ulocdata_set_no_substitute(uld, True)
        assert ulocdata_get_no_substitute(uld)



import pytest

from icupy.icu import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 58:
    pytest.skip("ICU4C<58", allow_module_level=True)

# fmt: off
from icupy.icu import (
    U_SHAPE_DIGITS_ALEN2AN_INIT_LR, U_SHAPE_LETTERS_UNSHAPE, UBIDI_LTR,
    UBIDI_MIRRORING_OFF, UBIDI_MIRRORING_ON, UBiDiOrder, u_shape_arabic,
    u_unescape, ubiditransform_close, ubiditransform_open,
    ubiditransform_transform,
)
from icupy.utils import gc

# fmt: on


def test_api():
    # UBiDiTransform *ubiditransform_open(UErrorCode *pErrorCode)
    # void ubiditransform_close(UBiDiTransform *pBidiTransform)
    with gc(ubiditransform_open(), ubiditransform_close) as transform:
        # From icu/source/test/cintltst/cbiditransformtst.c
        in_text = (
            "a[b]c \\u05d0(\\u05d1\\u05d2 \\u05d3)\\u05d4 1 d \\u0630 "
            "23\\u0660 e\\u06314 f \\ufeaf \\u0661\\u0662"
        )
        out_text = (
            "a[b]c 1 \\u05d4(\\u05d3 \\u05d2\\u05d1)\\u05d0 d 23\\u0660 "
            "\\u0630 e4\\u0631 f \\u0661\\u0662 \\ufeaf"
        )
        src = u_unescape(in_text)
        expected = u_unescape(out_text)

        # uint32_t ubiditransform_transform(
        #       UBiDiTransform *pBiDiTransform,
        #       const UChar *src,
        #       int32_t srcLength,
        #       UChar *dest,
        #       int32_t destSize,
        #       UBiDiLevel inParaLevel,
        #       UBiDiOrder inOrder,
        #       UBiDiLevel outParaLevel,
        #       UBiDiOrder outOrder,
        #       UBiDiMirroring doMirroring,
        #       uint32_t shapingOptions,
        #       UErrorCode *pErrorCode
        # )
        dest = ubiditransform_transform(
            transform,
            src,
            len(src),
            UBIDI_LTR,
            UBiDiOrder.UBIDI_LOGICAL,
            UBIDI_LTR,
            UBiDiOrder.UBIDI_VISUAL,
            UBIDI_MIRRORING_ON,
            0,
        )
        assert isinstance(dest, str)
        assert dest == expected

        out_text = (
            "a[b]c 1 \\u05d4)\\u05d3 \\u05d2\\u05d1(\\u05d0 d "
            "\\u0662\\u0663\\u0660 \\u0630 e\\u0664\\u0631 f \\u0661\\u0662 "
            "\\ufeaf"
        )
        expected = u_shape_arabic(
            u_unescape(out_text),
            -1,
            U_SHAPE_DIGITS_ALEN2AN_INIT_LR | U_SHAPE_LETTERS_UNSHAPE,
        )
        dest = ubiditransform_transform(
            transform,
            src,
            -1,
            UBIDI_LTR,
            UBiDiOrder.UBIDI_LOGICAL,
            UBIDI_LTR,
            UBiDiOrder.UBIDI_VISUAL,
            UBIDI_MIRRORING_OFF,
            U_SHAPE_DIGITS_ALEN2AN_INIT_LR | U_SHAPE_LETTERS_UNSHAPE,
        )
        assert isinstance(dest, str)
        assert dest == expected



from icupy.icu import CanonicalIterator, UnicodeString


def test_api():
    # From icu/source/test/intltest/canittst.cpp

    # icu::CanonicalIterator::CanonicalIterator(
    #       const UnicodeString &source,
    #       UErrorCode &status
    # )
    source = UnicodeString("ljubav")
    it = CanonicalIterator(source)

    # UnicodeString icu::CanonicalIterator::getSource()
    result = it.get_source()
    assert isinstance(result, UnicodeString)
    assert result == source

    # str -> UnicodeString
    source = "ljubav"
    it = CanonicalIterator(source)
    assert it.get_source() == source

    # void icu::CanonicalIterator::setSource(
    #       const UnicodeString &newSource,
    #       UErrorCode &status
    # )
    new_source = UnicodeString("\\u010d\\u017E").unescape()
    expected = [
        "c\u030cz\u030c",
        "c\u030c\u017e",
        "\u010dz\u030c",
        "\u010d\u017e",
    ]
    it.set_source(new_source)

    # __iter__() and __next__()
    assert len(list(it)) == len(expected)
    assert all(isinstance(x, UnicodeString) for x in it)
    assert all(x in it for x in expected)

    # void icu::CanonicalIterator::reset()
    it.reset()

    # UnicodeString icu::CanonicalIterator::next()
    result = it.next()
    assert isinstance(result, UnicodeString)
    assert not result.is_bogus()
    assert result in expected
    expected.remove(result)

    result = it.next()
    assert isinstance(result, UnicodeString)
    assert not result.is_bogus()
    assert result in expected
    expected.remove(result)

    result = it.next()
    assert isinstance(result, UnicodeString)
    assert not result.is_bogus()
    assert result in expected
    expected.remove(result)

    result = it.next()
    assert isinstance(result, UnicodeString)
    assert not result.is_bogus()
    assert result in expected
    expected.remove(result)

    result = it.next()
    assert isinstance(result, UnicodeString)
    assert result.is_bogus()
    assert len(expected) == 0

    # str -> UnicodeString
    new_source = "x\u0307\u0327"
    expected = ["x\u0307\u0327", "x\u0327\u0307", "\u1e8b\u0327"]
    it.set_source(new_source)
    assert len(list(it)) == len(expected)
    assert all(isinstance(x, UnicodeString) for x in it)
    assert all(x in it for x in expected)

    it.reset()
    result = it.next()
    assert isinstance(result, UnicodeString)
    assert not result.is_bogus()
    assert result in expected
    expected.remove(result)

    result = it.next()
    assert isinstance(result, UnicodeString)
    assert not result.is_bogus()
    assert result in expected
    expected.remove(result)

    result = it.next()
    assert isinstance(result, UnicodeString)
    assert not result.is_bogus()
    assert result in expected
    expected.remove(result)

    result = it.next()
    assert isinstance(result, UnicodeString)
    assert result.is_bogus()
    assert len(expected) == 0



from pathlib import Path

import pytest

# fmt: off
from icupy.icu import (
    U_ICU_VERSION_MAJOR_NUM, UnicodeSet, UnicodeString, UParseError,
    USpoofChecks, uspoof_are_confusable, uspoof_are_confusable_unicode_string,
    uspoof_are_confusable_utf8, uspoof_check, uspoof_check_unicode_string,
    uspoof_check_utf8, uspoof_clone, uspoof_close, uspoof_get_allowed_chars,
    uspoof_get_allowed_locales, uspoof_get_allowed_unicode_set,
    uspoof_get_checks, uspoof_get_skeleton, uspoof_get_skeleton_unicode_string,
    uspoof_get_skeleton_utf8, uspoof_open, uspoof_open_from_serialized,
    uspoof_open_from_source, uspoof_serialize, uspoof_set_allowed_chars,
    uspoof_set_allowed_locales, uspoof_set_allowed_unicode_set,
    uspoof_set_checks,
)
from icupy.utils import gc

# fmt: on


def test_api():
    # From icu/source/test/cintltst/spooftest.c

    # USpoofChecker *uspoof_open(UErrorCode *status)
    # void uspoof_close(USpoofChecker *sc)
    with gc(uspoof_open(), uspoof_close) as sc:
        sc_mixed = "\x73\u0441"
        sc_latin = "\x73\x63"

        # int32_t uspoof_areConfusable(
        #       const USpoofChecker *sc,
        #       const UChar *id1,
        #       int32_t length1,
        #       const UChar *id2,
        #       int32_t length2,
        #       UErrorCode *status
        # )
        assert (
            uspoof_are_confusable(sc, sc_latin, -1, sc_mixed, -1)
            == USpoofChecks.USPOOF_MIXED_SCRIPT_CONFUSABLE
        )

        s1 = UnicodeString("cxs")
        s2 = UnicodeString("\\u0441\\u0445\\u0455").unescape()

        # int32_t uspoof_areConfusableUnicodeString(
        #       const USpoofChecker *sc,
        #       const icu::UnicodeString &s1,
        #       const icu::UnicodeString &s2,
        #       UErrorCode *status
        # )
        assert uspoof_are_confusable_unicode_string(sc, s1, s2) == (
            USpoofChecks.USPOOF_MIXED_SCRIPT_CONFUSABLE
            | USpoofChecks.USPOOF_WHOLE_SCRIPT_CONFUSABLE
        )

        # int32_t uspoof_areConfusableUTF8(
        #       const USpoofChecker *sc,
        #       const char *id1,
        #       int32_t length1,
        #       const char *id2,
        #       int32_t length2,
        #       UErrorCode *status
        # )
        assert (
            uspoof_are_confusable_utf8(sc, sc_latin, -1, sc_mixed, -1)
            == USpoofChecks.USPOOF_MIXED_SCRIPT_CONFUSABLE
        )

        # int32_t uspoof_check(
        #       const USpoofChecker *sc,
        #       const UChar *id,
        #       int32_t length,
        #       int32_t *position,
        #       UErrorCode *status
        # )
        assert (
            uspoof_check(sc, sc_mixed, -1) == USpoofChecks.USPOOF_SINGLE_SCRIPT
        )
        assert (
            uspoof_check(sc, sc_mixed)  # length is optional
            == USpoofChecks.USPOOF_SINGLE_SCRIPT
        )

        # int32_t uspoof_checkUnicodeString(
        #       const USpoofChecker *sc,
        #       const icu::UnicodeString &id,
        #       int32_t *position,
        #       UErrorCode *status
        # )
        assert (
            uspoof_check_unicode_string(sc, UnicodeString(sc_mixed))
            == USpoofChecks.USPOOF_SINGLE_SCRIPT
        )

        # int32_t uspoof_checkUTF8(
        #       const USpoofChecker *sc,
        #       const char *id,
        #       int32_t length,
        #       int32_t *position,
        #       UErrorCode *status
        # )
        assert (
            uspoof_check_utf8(sc, sc_mixed, -1)
            == USpoofChecks.USPOOF_SINGLE_SCRIPT
        )
        assert (
            uspoof_check_utf8(sc, sc_mixed)  # length is optional
            == USpoofChecks.USPOOF_SINGLE_SCRIPT
        )

        # USpoofChecker *uspoof_clone(
        #       const USpoofChecker *sc,
        #       UErrorCode *status
        # )
        sc2 = uspoof_clone(sc)
        uspoof_close(sc2)

        # const USet *uspoof_getAllowedChars(
        #       const USpoofChecker *sc,
        #       UErrorCode *status
        # )
        uset = uspoof_get_allowed_chars(sc)
        uniset1 = UnicodeSet.from_uset(uset)
        assert uniset1.size() > 0

        # const char *uspoof_getAllowedLocales(
        #       USpoofChecker *sc,
        #       UErrorCode *status
        # )
        result = uspoof_get_allowed_locales(sc)
        assert isinstance(result, str)

        # const icu::UnicodeSet *uspoof_getAllowedUnicodeSet(
        #       const USpoofChecker *sc,
        #       UErrorCode *status
        # )
        uniset2 = uspoof_get_allowed_unicode_set(sc)
        assert isinstance(uniset2, UnicodeSet)
        assert uniset1 == uniset2

        # int32_t uspoof_getChecks(
        #       const USpoofChecker *sc,
        #       UErrorCode *status
        # )
        assert uspoof_get_checks(sc) == USpoofChecks.USPOOF_ALL_CHECKS

        lll_latin_a = "\x6c\x49\x31"
        lll_skel = "\x6c\x6c\x6c"

        # int32_t uspoof_getSkeleton(
        #       const USpoofChecker *sc,
        #       uint32_t type,
        #       const UChar *id,
        #       int32_t length,
        #       UChar *dest,
        #       int32_t destCapacity,
        #       UErrorCode *status
        # )
        type_ = USpoofChecks.USPOOF_ANY_CASE  # Deprecated in ICU 58
        dest = uspoof_get_skeleton(sc, type_, lll_latin_a, -1)
        assert isinstance(dest, str)
        assert dest == lll_skel

        # icu::UnicodeString &uspoof_getSkeletonUnicodeString(
        #       const USpoofChecker *sc,
        #       uint32_t type,
        #       const icu::UnicodeString &id,
        #       icu::UnicodeString &dest,
        #       UErrorCode *status
        # )
        dest = UnicodeString()
        result = uspoof_get_skeleton_unicode_string(
            sc, type_, UnicodeString(lll_latin_a), dest
        )
        assert isinstance(result, UnicodeString)
        assert id(result) == id(dest)
        assert dest == lll_skel

        # int32_t uspoof_getSkeletonUTF8(
        #       const USpoofChecker *sc,
        #       uint32_t type,
        #       const char *id,
        #       int32_t length,
        #       char *dest,
        #       int32_t destCapacity,
        #       UErrorCode *status
        # )
        dest = uspoof_get_skeleton_utf8(sc, type_, lll_latin_a, -1)
        assert isinstance(dest, str)
        assert dest == lll_skel

        # void uspoof_setAllowedLocales(
        #       USpoofChecker *sc,
        #       const char *localesList,
        #       UErrorCode *status
        # )
        uspoof_set_allowed_locales(sc, "en, ru_RU")
        assert uspoof_get_allowed_locales(sc) == "en, ru_RU"

        # void uspoof_setAllowedUnicodeSet(
        #       USpoofChecker *sc,
        #       const icu::UnicodeSet *chars,
        #       UErrorCode *status
        # )
        uspoof_set_allowed_unicode_set(sc, UnicodeSet(0, 0x10FFFF))
        assert uspoof_get_allowed_unicode_set(sc) == UnicodeSet(0, 0x10FFFF)

        # void uspoof_setChecks(
        #       USpoofChecker *sc,
        #       int32_t checks,
        #       UErrorCode *status
        # )
        uspoof_set_checks(
            sc,
            USpoofChecks.USPOOF_WHOLE_SCRIPT_CONFUSABLE
            | USpoofChecks.USPOOF_MIXED_SCRIPT_CONFUSABLE
            | USpoofChecks.USPOOF_ANY_CASE,
        )
        assert (
            uspoof_get_checks(sc)
            == USpoofChecks.USPOOF_WHOLE_SCRIPT_CONFUSABLE
            | USpoofChecks.USPOOF_MIXED_SCRIPT_CONFUSABLE
            | USpoofChecks.USPOOF_ANY_CASE
        )


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 51, reason="ICU4C<51")
def test_api_51():
    # fmt: off
    from icupy.icu import (
        URestrictionLevel, uspoof_get_inclusion_set,
        uspoof_get_inclusion_unicode_set, uspoof_get_recommended_set,
        uspoof_get_recommended_unicode_set, uspoof_get_restriction_level,
        uspoof_set_restriction_level,
    )

    # fmt: on
    # const USet *uspoof_getInclusionSet(UErrorCode *status)
    uset = uspoof_get_inclusion_set()
    uniset1 = UnicodeSet.from_uset(uset)
    assert uniset1.size() > 0

    # const icu::UnicodeSet *uspoof_getInclusionUnicodeSet(UErrorCode *status)
    uniset2 = uspoof_get_inclusion_unicode_set()
    assert isinstance(uniset2, UnicodeSet)
    assert uniset1 == uniset2

    # const USet *uspoof_getRecommendedSet(UErrorCode *status)
    uset = uspoof_get_recommended_set()
    uniset3 = UnicodeSet.from_uset(uset)
    assert uniset3.size() > 0

    # const icu::UnicodeSet *uspoof_getRecommendedUnicodeSet(
    #       UErrorCode *status
    # )
    uniset4 = uspoof_get_recommended_unicode_set()
    assert isinstance(uniset4, UnicodeSet)
    assert uniset3 == uniset4

    with gc(uspoof_open(), uspoof_close) as sc:
        # URestrictionLevel uspoof_getRestrictionLevel(const USpoofChecker *sc)
        assert (
            uspoof_get_restriction_level(sc)
            == URestrictionLevel.USPOOF_HIGHLY_RESTRICTIVE
        )

        # void uspoof_setRestrictionLevel(
        #       USpoofChecker *sc,
        #       URestrictionLevel restrictionLevel
        # )
        uspoof_set_restriction_level(
            sc, URestrictionLevel.USPOOF_MODERATELY_RESTRICTIVE
        )
        assert (
            uspoof_get_restriction_level(sc)
            == URestrictionLevel.USPOOF_MODERATELY_RESTRICTIVE
        )


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 58, reason="ICU4C<58")
def test_api_58():
    # From icu/source/test/intltest/itspoof.cpp
    # fmt: off
    from icupy.icu import (
        URestrictionLevel, uspoof_check2, uspoof_check2_unicode_string,
        uspoof_check2_utf8, uspoof_close_check_result,
        uspoof_get_check_result_checks, uspoof_get_check_result_numerics,
        uspoof_get_check_result_restriction_level, uspoof_open_check_result,
    )

    # fmt: on
    # USpoofCheckResult *uspoof_openCheckResult(UErrorCode *status)
    # void uspoof_closeCheckResult(USpoofCheckResult *checkResult)
    with gc(uspoof_open(), uspoof_close) as sc, gc(
        uspoof_open_check_result(), uspoof_close_check_result
    ) as check_result:
        # int32_t uspoof_check2(
        #       const USpoofChecker *sc,
        #       const UChar *id,
        #       int32_t length,
        #       USpoofCheckResult *checkResult,
        #       UErrorCode *status
        # )
        assert uspoof_check2(sc, "m\u0307", -1, check_result) == 0
        assert uspoof_check2(sc, "m\u0307", -1, None) == 0
        assert uspoof_check2(sc, "m\u0307", -1) == 0  # checkResult is optional
        assert uspoof_check2(sc, "m\u0307") == 0  # length is optional

        # int32_t uspoof_check2UnicodeString(
        #       const USpoofChecker *sc,
        #       const icu::UnicodeString &id,
        #       USpoofCheckResult *checkResult,
        #       UErrorCode *status
        # )
        assert (
            uspoof_check2_unicode_string(
                sc, UnicodeString("m\u0307"), check_result
            )
            == 0
        )
        assert (
            uspoof_check2_unicode_string(sc, UnicodeString("m\u0307"), None)
            == 0
        )
        assert (
            uspoof_check2_unicode_string(sc, UnicodeString("m\u0307")) == 0
        )  # checkResult is optional

        # int32_t uspoof_check2UTF8(
        #       const USpoofChecker *sc,
        #       const char *id,
        #       int32_t length,
        #       USpoofCheckResult *checkResult,
        #       UErrorCode *status
        # )
        assert uspoof_check2_utf8(sc, "m\u0307", -1, check_result) == 0
        assert uspoof_check2_utf8(sc, "m\u0307", -1, None) == 0
        assert (
            uspoof_check2_utf8(sc, "m\u0307", -1) == 0
        )  # checkResult is optional
        assert uspoof_check2_utf8(sc, "m\u0307") == 0  # length is optional

        # int32_t uspoof_getCheckResultChecks(
        #       const USpoofCheckResult *checkResult,
        #       UErrorCode *status
        # )
        assert uspoof_get_check_result_checks(check_result) == 0

        assert (
            uspoof_check2_utf8(sc, "i\u0307", -1, check_result)
            == USpoofChecks.USPOOF_HIDDEN_OVERLAY
        )
        assert (
            uspoof_get_check_result_checks(check_result)
            == USpoofChecks.USPOOF_HIDDEN_OVERLAY
        )

        # const USet *uspoof_getCheckResultNumerics(
        #       const USpoofCheckResult *checkResult,
        #       UErrorCode *status
        # )
        uspoof_set_checks(
            sc,
            USpoofChecks.USPOOF_MIXED_NUMBERS
            | USpoofChecks.USPOOF_RESTRICTION_LEVEL,
        )
        assert (
            uspoof_check2_unicode_string(
                sc, UnicodeString("\u0967"), check_result
            )
            == 0
        )
        uset = uspoof_get_check_result_numerics(check_result)
        uniset = UnicodeSet.from_uset(uset)
        assert uniset == UnicodeSet("[\u0966]")

        # URestrictionLevel uspoof_getCheckResultRestrictionLevel(
        #       const USpoofCheckResult *checkResult,
        #       UErrorCode *status
        # )
        assert (
            uspoof_get_check_result_restriction_level(check_result)
            == URestrictionLevel.USPOOF_SINGLE_SCRIPT_RESTRICTIVE
        )


def test_open_from_serialized():
    with gc(uspoof_open(), uspoof_close) as sc:
        # int32_t uspoof_serialize(
        #       USpoofChecker *sc,
        #       void *data,
        #       int32_t capacity,
        #       UErrorCode *status
        # )
        data = uspoof_serialize(sc)
        assert isinstance(data, bytes)
        assert len(data) > 0

        # USpoofChecker *uspoof_openFromSerialized(
        #       const void *data,
        #       int32_t length,
        #       int32_t *pActualLength,
        #       UErrorCode *pErrorCode
        # )
        with gc(
            uspoof_open_from_serialized(data, len(data)), uspoof_close
        ) as sc2:
            uniset1 = uspoof_get_allowed_unicode_set(sc)
            uniset2 = uspoof_get_allowed_unicode_set(sc2)
            assert uniset1 == uniset2

        with gc(
            uspoof_open_from_serialized(data),  # length is optional
            uspoof_close,
        ) as sc2:
            uniset1 = uspoof_get_allowed_unicode_set(sc)
            uniset2 = uspoof_get_allowed_unicode_set(sc2)
            assert uniset1 == uniset2


def test_open_from_source():
    parent = Path(__file__).resolve().parent
    path1 = parent / "confusables.txt"
    path2 = parent / "confusablesWholeScript.txt"
    try:
        with open(path1, "rb") as fp1, open(path2, "rb") as fp2:
            confusables = fp1.read()
            confusables_whole_script = fp2.read()
            good_latin = "\x75\x7a"

            # USpoofChecker *uspoof_openFromSource(
            #       const char *confusables,
            #       int32_t confusablesLen,
            #       const char *confusablesWholeScript,
            #       int32_t confusablesWholeScriptLen,
            #       int32_t *errType,
            #       UParseError *pe,
            #       UErrorCode *status
            # )
            pe = UParseError()
            with gc(
                uspoof_open_from_source(
                    confusables,
                    len(confusables),
                    confusables_whole_script,
                    len(confusables_whole_script),
                    pe,
                ),
                uspoof_close,
            ) as sc:
                assert pe.offset == 0
                assert uspoof_check(sc, good_latin, -1) == 0

            pe = UParseError()
            with gc(
                uspoof_open_from_source(
                    confusables, -1, confusables_whole_script, -1, pe
                ),
                uspoof_close,
            ) as sc:
                assert pe.offset == 0
                assert uspoof_check(sc, good_latin, -1) == 0

            pe = UParseError()
            with gc(
                uspoof_open_from_source(
                    confusables, -1, None, 0, pe  # Deprecated in ICU 58
                ),
                uspoof_close,
            ) as sc:
                assert pe.offset == 0
                assert uspoof_check(sc, good_latin, -1) == 0
    except FileNotFoundError:
        pytest.skip(
            "confusables.txt or confusablesWholeScript.txt, "
            "or both files are not found (not an error). "
            "You need to copy them from "
            "<icu4c>/icu/source/data/unidata/"
        )


def test_set_allowed_chars():
    with gc(uspoof_open(), uspoof_close) as sc:
        # void uspoof_setAllowedChars(
        #       USpoofChecker *sc,
        #       const USet *chars,
        #       UErrorCode *status
        # )
        uniset = UnicodeSet(0x41, 0x5A)
        chars = uniset.to_uset()
        uspoof_set_allowed_chars(sc, chars)  # USet*
        uniset4 = uspoof_get_allowed_unicode_set(sc)
        assert uniset4 == UnicodeSet("[A-Z]")


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 63, reason="ICU4C<63")
def test_set_allowed_chars_63():
    from icupy.icu import UProperty, u_get_binary_property_set

    with gc(uspoof_open(), uspoof_close) as sc:
        # const USet *u_getBinaryPropertySet(
        #       UProperty property,
        #       UErrorCode *pErrorCode
        # )
        chars = u_get_binary_property_set(UProperty.UCHAR_POSIX_BLANK)

        # void uspoof_setAllowedChars(
        #       USpoofChecker *sc,
        #       const USet *chars,
        #       UErrorCode *status
        # )
        uspoof_set_allowed_chars(sc, chars)  # const USet*
        uniset = uspoof_get_allowed_unicode_set(sc)
        assert repr(uniset) == (
            "UnicodeSet(["
            "\\u0009\\ \\u00A0\\u1680\\u2000-\\u200A\\u202F\\u205F\\u3000"
            "])"
        )



import copy
import sys

import pytest

# fmt: off
from icupy.icu import (
    U_ICU_VERSION_MAJOR_NUM, DateFormat, ErrorCode, FieldPosition,
    FieldPositionIterator, GregorianCalendar, ICUError, Locale, MessagePattern,
    ParsePosition, TimeZone, UErrorCode, UnicodeString, UParseError, u_failure,
    u_get_version, u_success, u_version_from_string, u_version_to_string,
)

# fmt: on


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_constrained_field_position():
    from icupy.icu import ConstrainedFieldPosition, ListFormatter

    # icu::ConstrainedFieldPosition
    cfpos = ConstrainedFieldPosition()

    # ConstrainedFieldPosition.__repr__() -> str
    fmt = ListFormatter.create_instance(Locale.get_english())
    items = [
        UnicodeString("hello"),
        UnicodeString("wonderful"),
        UnicodeString("world"),
    ]
    fl = fmt.format_strings_to_value(items, len(items))
    assert fl.next_position(cfpos)
    assert repr(cfpos) == (
        "ConstrainedFieldPosition("
        "category=0x1003, field=0, start=0, limit=5"
        ")"
    )


def test_deprecated_api():
    # icupy.number module is deprecated
    with pytest.warns(UserWarning):
        import icupy.number  # noqa: F401


def test_field_position():
    # icu::FieldPosition
    # FieldPosition.__repr__() -> str
    fmt = DateFormat.create_date_time_instance(
        DateFormat.MEDIUM, DateFormat.SHORT, Locale.get_english()
    )
    fmt.set_time_zone(TimeZone.get_gmt())
    append_to = UnicodeString()
    pos_iter = FieldPositionIterator()
    fmt.format(1215298800000, append_to, pos_iter)
    fpos1 = FieldPosition()
    assert pos_iter.next(fpos1)
    assert repr(fpos1) == "FieldPosition(field=2, begin_index=0, end_index=3)"

    # FieldPosition *icu::FieldPosition::clone()
    # FieldPosition.__copy__() -> FieldPosition
    # FieldPosition.__deepcopy__(Optional[memo]) -> FieldPosition
    # FieldPosition.__eq__(other) -> bool
    # FieldPosition.__ne__(other) -> bool
    fpos2 = FieldPosition()
    fpos1a = fpos1.clone()
    assert isinstance(fpos1a, FieldPosition)
    fpos1b = copy.copy(fpos1)
    fpos1c = copy.deepcopy(fpos1)
    assert fpos1 == fpos1a == fpos1b == fpos1c
    assert fpos1 != fpos2


def test_icu_error():
    assert issubclass(ICUError, Exception)

    with pytest.raises(ICUError) as exc_info:
        _ = u_version_to_string([1, 2, 3])
    ex = exc_info.value
    assert isinstance(ex, ICUError)
    assert len(ex.args) == 1
    error_code = ex.args[0]
    assert isinstance(error_code, ErrorCode)

    # const char *icu::ErrorCode::errorName()
    error_name = error_code.error_name
    assert isinstance(error_name, str)
    assert error_name == "U_ILLEGAL_ARGUMENT_ERROR"

    # UErrorCode icu::ErrorCode::get()
    status = error_code.get()
    assert isinstance(status, UErrorCode)
    assert status == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    # ErrorCode.__eq__(self, other: UErrorCode) -> bool
    assert error_code == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    if U_ICU_VERSION_MAJOR_NUM >= 63:
        with pytest.raises(ICUError) as exc_info:
            _ = Locale.for_language_tag("x")
        ex = exc_info.value
        assert isinstance(ex, ICUError)
        assert len(ex.args) == 2
        assert isinstance(ex.args[0], ErrorCode)  # icu::ErrorCode
        assert isinstance(ex.args[1], str)  # An error message

    # ErrorCode.__repr__() -> str
    assert repr(error_code) == "ErrorCode(<U_ILLEGAL_ARGUMENT_ERROR: 1>)"


# TODO: Remove test_icu_exception().
def test_icu_exception():
    """Backward compatibility test.
    ICUException is deprecated. Use ICUError instead.
    """

    from icupy import ICUException

    assert issubclass(ICUException, Exception)

    with pytest.raises(ICUException) as exc_info:
        _ = u_version_to_string([1, 2, 3])
    ex = exc_info.value
    assert isinstance(ex, ICUException)
    assert len(ex.args) == 1
    error_code = ex.args[0]
    assert isinstance(error_code, ErrorCode)

    # const char *icu::ErrorCode::errorName()
    error_name = error_code.error_name
    assert isinstance(error_name, str)
    assert error_name == "U_ILLEGAL_ARGUMENT_ERROR"

    # UErrorCode icu::ErrorCode::get()
    status = error_code.get()
    assert isinstance(status, UErrorCode)
    assert status == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    # ErrorCode.__eq__(self, other: UErrorCode) -> bool
    assert error_code == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    if U_ICU_VERSION_MAJOR_NUM >= 63:
        with pytest.raises(ICUException) as exc_info:
            _ = Locale.for_language_tag("x")
        ex = exc_info.value
        assert isinstance(ex, ICUException)
        assert len(ex.args) == 2
        assert isinstance(ex.args[0], ErrorCode)  # icu::ErrorCode
        assert isinstance(ex.args[1], str)  # Error message


@pytest.mark.skipif(
    not sys.platform.startswith("win"), reason="On Windows only"
)
def test_import():
    import importlib
    import os
    import tempfile

    import icupy

    old_value = os.environ.get("ICU_ROOT")
    try:
        with tempfile.TemporaryDirectory() as pathname:
            os.environ["ICU_ROOT"] = pathname
            with pytest.raises(FileNotFoundError):
                importlib.reload(icupy)
    finally:
        if old_value is not None:
            os.environ["ICU_ROOT"] = old_value
        else:
            del os.environ["ICU_ROOT"]


def test_parse_position():
    # icu::ParsePosition
    pos1 = ParsePosition()
    pos2 = ParsePosition(1)
    pos3 = ParsePosition(pos1)

    # bool icu::ParsePosition::operator!=(const ParsePosition &that)
    assert pos1 != pos2
    assert not (pos1 != pos3)
    assert pos2 != pos3

    # bool icu::ParsePosition::operator==(const ParsePosition &that)
    assert not (pos1 == pos2)
    assert pos1 == pos3
    assert not (pos2 == pos3)

    # ParsePosition *icu::ParsePosition::clone()
    # ParsePosition.__copy__() -> ParsePosition
    # ParsePosition.__deepcopy__(Optional[memo]) -> ParsePosition
    pos4 = pos1.clone()
    assert isinstance(pos4, ParsePosition)
    pos4a = copy.copy(pos1)
    pos4b = copy.deepcopy(pos1)
    assert pos1 == pos4 == pos4a == pos4b

    # void icu::ParsePosition::setErrorIndex(int32_t ei)
    assert pos1.get_error_index() == -1
    pos1.set_error_index(8)
    assert pos1.get_error_index() == 8

    # void icu::ParsePosition::setIndex(int32_t index)
    assert pos1.get_index() == 0
    pos1.set_index(4)
    assert pos1.get_index() == 4

    # ParsePosition.__repr__() -> str
    fmt = DateFormat.create_date_time_instance(
        DateFormat.SHORT, DateFormat.LONG, Locale.get_english()
    )
    zone = TimeZone.create_time_zone("PST")
    cal = GregorianCalendar(zone)
    cal.clear()
    pos = ParsePosition()
    fmt.parse(UnicodeString("07/10/96 4:5:0 PM PDT"), cal, pos)
    assert pos.get_error_index() == -1
    assert repr(pos) == "ParsePosition(index=21, error_index=-1)"

    pos = ParsePosition()
    fmt.parse(UnicodeString("07/10/96 4:5:0 PM JST"), cal, pos)
    assert pos.get_error_index() == 18
    assert repr(pos) == "ParsePosition(index=0, error_index=18)"


def test_u_failure():
    assert not u_failure(UErrorCode.U_USING_FALLBACK_WARNING)
    assert not u_failure(UErrorCode.U_ZERO_ERROR)
    assert u_failure(UErrorCode.U_ILLEGAL_ARGUMENT_ERROR)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 49, reason="ICU4C<49")
def test_u_get_data_version():
    from icupy.icu import u_get_data_version

    # void u_getDataVersion(
    #       UVersionInfo dataVersionFillin,
    #       UErrorCode *status
    # )
    data_version = u_get_data_version()
    assert isinstance(data_version, list)
    assert len(data_version) == 4
    assert all(isinstance(x, int) for x in data_version)
    assert all(x >= 0 for x in data_version)


def test_u_get_version():
    # void u_getVersion(UVersionInfo versionArray)
    version_array = u_get_version()
    assert isinstance(version_array, list)
    assert len(version_array) == 4
    assert all(isinstance(x, int) for x in version_array)
    assert all(x >= 0 for x in version_array)


def test_u_success():
    assert u_success(UErrorCode.U_USING_FALLBACK_WARNING)
    assert u_success(UErrorCode.U_ZERO_ERROR)
    assert not u_success(UErrorCode.U_ILLEGAL_ARGUMENT_ERROR)


def test_u_version_from_string():
    # void u_versionFromString(
    #       UVersionInfo versionArray,
    #       const char *versionString
    # )
    version_array = u_version_from_string("123.45.67.89")
    assert isinstance(version_array, list)
    assert len(version_array) == 4
    assert version_array[0] == 123
    assert version_array[1] == 45
    assert version_array[2] == 67
    assert version_array[3] == 89


def test_u_version_to_string():
    # void u_versionToString(
    #       const UVersionInfo versionArray,
    #       char *versionString
    # )
    version_string = u_version_to_string([123, 45, 67, 89])
    assert isinstance(version_string, str)
    assert version_string == "123.45.67.89"

    with pytest.raises(ICUError) as exc_info:
        _ = u_version_to_string([1, 2, 3])
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    with pytest.raises(ICUError) as exc_info:
        _ = u_version_to_string([1, 2, 3, 4, 5])
    assert exc_info.value.args[0] == UErrorCode.U_ILLEGAL_ARGUMENT_ERROR

    with pytest.raises(TypeError):
        _ = u_version_to_string([256, 0, 0, 0])  # [0, 255]


def test_uparse_error():
    # UParseError.__repr__() -> str
    pattern = UnicodeString("I see {many}")
    parse_error = UParseError()
    _ = MessagePattern(pattern, parse_error)
    assert parse_error.offset == 0
    assert repr(parse_error) == (
        "UParseError(line=0, offset=0, pre_context='', post_context='')"
    )

    pattern = UnicodeString("I see {'many'}")
    parse_error = UParseError()
    with pytest.raises(ICUError):
        _ = MessagePattern(pattern, parse_error)
    assert parse_error.offset == 7
    assert repr(parse_error) == (
        "UParseError("
        "line=0, offset=7, pre_context='I see {', post_context='\\'many\\'}'"
        ")"
    )



import sys
from contextlib import contextmanager
from typing import Any, Iterator

if sys.version_info[:2] >= (3, 9):
    from collections.abc import Callable
else:
    from typing import Callable

__all__ = ["gc"]


@contextmanager
def gc(obj: Any, closer: Callable[[Any], Any]) -> Iterator[Any]:
    """Context to automatically close something at the end of a block.

    Code like this:

    .. code-block:: python

        with gc(<module>.open(<arguments>), <module>.close) as f:
            <block>

    is equivalent to this:

    .. code-block:: python

        f = <module>.open(<arguments>)
        try:
            <block>
        finally:
            <module>.close(f)
    """
    assert callable(closer)
    try:
        yield obj
    finally:
        closer(obj)



"""
Module for icu::number namespace
"""

import warnings

from .icu.number import *  # noqa

# TODO: Remove src/icupy/number.py in a future release.
warnings.warn(
    "icupy.number module is deprecated. Use icupy.icu.number instead.",
    stacklevel=2,
)



import pytest

# fmt: off
from icupy.icu import (
    U_ICU_VERSION_MAJOR_NUM, UnicodeSet, UnicodeSetIterator, UnicodeString,
)

# fmt: on


def test_next():
    # [1]
    # icu::UnicodeSetIterator::UnicodeSetIterator(const UnicodeSet &set)
    uniset = UnicodeSet("[a\\U0001abcd{ab}]")
    it = UnicodeSetIterator(uniset)

    # UBool icu::UnicodeSetIterator::next()
    assert it.next()

    # UBool icu::UnicodeSetIterator::isString()
    assert not it.is_string()

    # UChar32 icu::UnicodeSetIterator::getCodepoint()
    assert it.get_codepoint() == ord("a")

    # const UnicodeString &icu::UnicodeSetIterator::getString()
    result = it.get_string()
    assert isinstance(result, UnicodeString)
    assert result == "a"

    assert it.next()
    assert not it.is_string()
    assert it.get_codepoint() == 0x1ABCD
    assert it.get_string() == "\U0001abcd"

    assert it.next()
    assert it.is_string()
    assert it.get_string() == "ab"

    assert not it.next()

    # [1]
    # void icu::UnicodeSetIterator::reset()
    it.reset()
    assert it.next()
    assert not it.is_string()
    assert it.get_codepoint() == ord("a")
    assert it.get_string() == "a"


def test_next_range():
    # [2]
    # icu::UnicodeSetIterator::UnicodeSetIterator()
    it = UnicodeSetIterator()

    # UBool icu::UnicodeSetIterator::nextRange()
    assert not it.next_range()

    # [2]
    # void icu::UnicodeSetIterator::reset(const UnicodeSet &set)
    uniset = UnicodeSet("[a0-9{ab}]")
    it.reset(uniset)

    assert it.next_range()
    assert not it.is_string()
    assert it.get_codepoint() == ord("0")

    # UChar32 icu::UnicodeSetIterator::getCodepointEnd()
    assert it.get_codepoint_end() == ord("9")

    assert it.next_range()
    assert not it.is_string()
    assert it.get_codepoint() == ord("a")
    assert it.get_codepoint_end() == ord("a")

    assert it.next_range()
    assert it.is_string()
    assert it.get_string() == "ab"

    assert not it.next_range()


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 70, reason="ICU4C<70")
def test_skip_to_strings():
    uniset = UnicodeSet("[a0-9{ab}]")
    it = UnicodeSetIterator(uniset)

    # UnicodeSetIterator &icu::UnicodeSetIterator::skipToStrings()
    result = it.skip_to_strings()
    assert isinstance(result, UnicodeSetIterator)
    assert id(result) == id(it)

    assert it.next()
    assert it.is_string()
    assert it.get_string() == "ab"

    assert not it.next()
